#define EX2_C

#include "main.h"

#include <gtk/gtk.h>

#define UPDATE_SECS 1

#define FD_SZ 10

static unsigned int offs[FD_SZ];

static unsigned int add_write(void)
{
 int fd = open("ex2_test_write", O_WRONLY|O_CREAT);
 unsigned int ret = 0;

 unlink("ex2_test_write"); /* just have the fd */

 if (fd  == -1)
 {
  fprintf(stderr, " Err: %d %s\n", errno, strerror(errno));
  exit (1);
 }

 fprintf(stdout, "write: open(%d)\n", fd);

 ret = socket_poll_add(fd);
 if (!ret)
   exit (10);

 if (rand() & 4)
   SOCKET_POLL_INDICATOR(ret)->events = POLLOUT;

 return (ret);
}

static unsigned int add_read(void)
{
 int fd = open("ex2.c", O_RDONLY);
 unsigned int ret = 0;
 
 if (fd  == -1)
 {
  fprintf(stderr, " Err: %d %s\n", errno, strerror(errno));
  exit (1);
 }

 fprintf(stdout, "read:  open(%d)\n", fd);

 ret = socket_poll_add(fd);
 if (!ret)
   exit (8);

 if (rand() & 4)
   SOCKET_POLL_INDICATOR(ret)->events = POLLIN;
 
 return (ret);
}

static void setup(void)
{
 unsigned int count = 0;

 while (count < FD_SZ)
 {
  offs[count] = add_write();
  
  ++count;
 }
}

static void go_through_polls(void)
{
 unsigned int count = 0;

 while (count < FD_SZ)
 {
  char buffer[256] = "abcd\n";
  ssize_t len = 0;

  if (offs[count])
  {
   if (SOCKET_POLL_INDICATOR(offs[count])->revents & (POLLERR|POLLHUP|POLLNVAL))
   {
    fprintf(stdout, "inval %d\n", SOCKET_POLL_INDICATOR(offs[count])->fd);
    close(SOCKET_POLL_INDICATOR(offs[count])->fd);
    socket_poll_del(offs[count]);
    offs[count] = 0;
   }
   else if (SOCKET_POLL_INDICATOR(offs[count])->revents & POLLIN)
   {
    len = read(SOCKET_POLL_INDICATOR(offs[count])->fd, buffer, sizeof(buffer));
    if (len == -1)
    {
     if (errno == EBADF)
       fprintf(stdout, "EBADF: read(%d)\n", SOCKET_POLL_INDICATOR(offs[count])->fd);
     else
       fprintf(stdout, "%d: read(%d)\n", errno, SOCKET_POLL_INDICATOR(offs[count])->fd);
     close(SOCKET_POLL_INDICATOR(offs[count])->fd);
     socket_poll_del(offs[count]);
     offs[count] = 0;
    }
    else if (len == 0)
    {
     fprintf(stdout, "done:  read(%d)\n", SOCKET_POLL_INDICATOR(offs[count])->fd);
     close(SOCKET_POLL_INDICATOR(offs[count])->fd);
     socket_poll_del(offs[count]);
     offs[count] = 0;
    }
   }
   else if (SOCKET_POLL_INDICATOR(offs[count])->revents & POLLOUT)
   {
    len = write(SOCKET_POLL_INDICATOR(offs[count])->fd, buffer, strlen(buffer));
    if (len == -1)
    {
     if (errno == EBADF)
       fprintf(stdout, "EBADF: write(%d)\n", SOCKET_POLL_INDICATOR(offs[count])->fd);
     else
       fprintf(stdout, "%d: write(%d)\n", errno, SOCKET_POLL_INDICATOR(offs[count])->fd);
     close(SOCKET_POLL_INDICATOR(offs[count])->fd);
     socket_poll_del(offs[count]);
     offs[count] = 0;
    }
   }
  }
  
  ++count;
 }
}

static void do_something(void)
{
 unsigned int count = 0;

 while (count < FD_SZ)
 {
  int ra = rand();
  
  if (!offs[count])
  {
   if (ra & 2)
   {
    if ((ra & 7) ==  7)
      offs[count] = add_write();
    else
      offs[count] = add_read();
   }
  }
  else if ((ra & 3) == 3)
  {
   if ((ra & 12) == 12)
   {
    fprintf(stdout, "done:  close(%d)\n", SOCKET_POLL_INDICATOR(offs[count])->fd);
    close(SOCKET_POLL_INDICATOR(offs[count])->fd);
    socket_poll_del(offs[count]);
    offs[count] = 0;
   }
   else if (0)
   {
    fprintf(stdout, "pre: close(%d)\n", SOCKET_POLL_INDICATOR(offs[count])->fd);
    close(SOCKET_POLL_INDICATOR(offs[count])->fd);
   }
  }
  
  ++count;
 }
}


/* gtk stuff... */

/* Get the selected filename and print it to the console */
void file_ok_sel (GtkWidget __attribute__ ((__unused__)) *w,
                  GtkFileSelection *fs)
{
 g_print("%s\n", gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));
}

void destroy (GtkWidget __attribute__ ((__unused__)) *widget,
              gpointer __attribute__ ((__unused__)) *data)
{
 gtk_main_quit();
}

static void parse_rc_files(void)
{
 char buf[1050];

 snprintf(buf, sizeof(buf), "%s/.gtkrc", getenv("HOME"));
 
 gtk_rc_parse(buf);
}

void poll_cb(int __attribute__ ((__unused__)) fds_have_state)
{
 if (fds_have_state == -1)
   return;

 go_through_polls();
 
 if (rand() & 2)
   do_something();
}

int main(int argc, char *argv[])
{
 GtkWidget *filew;

 socket_poll_init(0, SOCKET_POLL_TYPE_MAP_COMPRESSED);
 
 gtk_init (&argc, &argv);

 parse_rc_files();
 
 /* Create a new file selection widget */
 filew = gtk_file_selection_new ("File selection");
 
 gtk_signal_connect (GTK_OBJECT (filew), "destroy",
                     (GtkSignalFunc) destroy, &filew);
 /* Connect the ok_button to file_ok_sel function */
 gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
                     "clicked", (GtkSignalFunc) file_ok_sel, filew );
 
 /* Connect the cancel_button to destroy the widget */
 gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(filew)->cancel_button),                           "clicked", (GtkSignalFunc) gtk_widget_destroy,
                           GTK_OBJECT (filew));
 
 /* Lets set the filename, as if this were a save dialog, and we are giving
    a default filename */
 gtk_file_selection_set_filename(GTK_FILE_SELECTION(filew), 
                                 argc ? argv[argc - 1] : "");

 srand(getpid() ^ time(0));

 setup();
 
 /* these 2 are the magic that makes it work with gtk */
 socket_poll_cntl_opt(SOCKET_POLL_CNTL_OPT_SET_OVERLOAD_FUNC, poll_cb);
 g_main_set_poll_func(socket_poll_update_overload);
 
 gtk_widget_show(filew);
 gtk_main();

 exit (EXIT_SUCCESS);
}
